# -*- coding: utf-8 -*-
"""
Настройка пользовательского интерфейса
(c)2012 ЗАО "РИПАС" и Демин Федор,
www.ripas.ru dff@ripas.ru
"""

from PythonQt import Qt, QtCore, QtGui
from RoadCentre import kernel

def widgetFromUi(uiFileName):
    f = Qt.QFile(uiFileName)
    if f.open(f.ReadOnly):
        from PythonQt.QtUiTools import QUiLoader
        ui_loader = QUiLoader()
        if hasattr(kernel,'custom_plugins_path'):
            ui_loader.addPluginPath(kernel.custom_plugins_path)
        w = ui_loader.load(f)
        return w
    return None


def graphicsViewMouseMove(p):
    x = p.x()
    y = p.y()

    latDeg = int(y)
    y = (y - latDeg) * 60
    latMin = int(y)
    latMinF = int((y - latMin) * 10000)

    lonDeg = int(x)
    x = (x - lonDeg) * 60
    lonMin = int(x)
    lonMinF = int((x - lonMin) * 10000)

    mainWindow.statusBar().showMessage(
        u'%02dВ°%02d.%04dN %02dВ°%02d.%04dE (%f,%f)' %
        (latDeg, latMin, latMinF, lonDeg, lonMin, lonMinF, p.y(), p.x()))


# mainWindow.statusBar().showMessage('%f %f'%(x, y))

def graphicsViewMouseMove2(p):
    x = p.x()
    y = p.y()

    latDeg = int(y)
    y = (y - latDeg) * 60
    latMin = int(y)
    y = (y - latMin) * 60
    latSec = int(y)
    latSecF = int((y - latSec) * 100)

    lonDeg = int(x)
    x = (x - lonDeg) * 60
    lonMin = int(x)
    x = (x - lonMin) * 60
    lonSec = int(x)
    lonSecF = int((x - lonSec) * 100)

    mainWindow.statusBar().showMessage(
        u'%02dВ°%02d\'%02d.%02d"N %02dВ°%02d\'%02d.%02d"E (%f,%f)' %
        (latDeg, latMin, latSec, latSecF, lonDeg, lonMin, lonSec, lonSecF, p.y(), p.x()))
    mainWindow.statusBar().showMessage('%f %f' % (x, y))


def msgAdd(msg, color=0, alt_console=None):
    output = alt_console
    if not output:
        try:
            output = console
        except:
            return
    cursor = output.textCursor()
    cursor.movePosition(cursor.End)
    fr = output.currentCharFormat()
    fr.setForeground(QtGui.QBrush(color))
    cursor.insertText(msg, fr)
    output.setTextCursor(cursor)


def msgNew(msg, color=0, alt_console=None, msg_time=None):
    output = alt_console
    if not output:
        try:
            output = console
        except:
            return
    cursor = output.textCursor()
    cursor.movePosition(cursor.End)
    fr = output.currentCharFormat()
    fr.setForeground(QtGui.QBrush(color))
    if not msg.endswith('\n'):
        msg+='\n'
    import time
    if not msg_time:
        cursor.insertText(time.strftime('%X ') + msg,fr)
    else:
        cursor.insertText(msg_time + ' ' + msg,fr)
    cursor.movePosition(cursor.StartOfLine)
    output.setTextCursor(cursor)


def msgInfo(msg, alt_console=None, msg_time=None):
    msgNew(msg, QtCore.Qt.darkBlue, alt_console, msg_time)


def msgOk(msg, alt_console=None, msg_time=None):
    msgNew(msg, QtCore.Qt.darkGreen, alt_console, msg_time)


def msgWarning(msg, alt_console=None, msg_time=None):
    msgNew(msg, QtCore.Qt.darkYellow, alt_console, msg_time)


def msgError(msg, alt_console=None, msg_time=None):
    msgNew(msg, QtCore.Qt.red, alt_console, msg_time)


def msgDebug(msg, alt_console=None, msg_time=None):
    msgNew(msg, QtCore.Qt.darkCyan, alt_console, msg_time)


def mainWindowFileName():
    d = QtCore.QDir.home()
    app = QtCore.QCoreApplication.instance()

    from sys import platform
    if platform == 'win32':
        subdir = 'Application Data/%s/%s/cashe/' % (app.organizationName, app.applicationName)
    else:
        subdir = '.cache/%s/%s/' % (app.organizationName, app.applicationName)
    d.mkpath(subdir)
    d.cd(subdir)
    return d.absoluteFilePath('mainWindow_qt5')


def mainWindowSave():
    from PythonQt.Qt import QFile
    from settings import userSettings
    b = mainWindow.saveGeometry()
    f = QFile(mainWindowFileName() + '.geom')
    f.open(QFile.WriteOnly)
    f.write(b)
    f.close()
    b = mainWindow.saveState()
    f = QFile(mainWindowFileName() + '.state')
    f.open(QFile.WriteOnly)
    f.write(b)
    f.close()
    if 'treeView' in globals():
        tvh=treeView.header()
        if tvh:
            userSettings.setValue('TreeViewState/state',tvh.saveState())
            userSettings.sync()


def mainWindowResore():
    from PythonQt.Qt import QFile
    from settings import userSettings
    f = QFile(mainWindowFileName() + '.geom')
    f.open(QFile.ReadOnly)
    b = f.readAll()
    f.close()
    mainWindow.restoreGeometry(b)
    f = QFile(mainWindowFileName() + '.state')
    f.open(QFile.ReadOnly)
    b = f.readAll()
    f.close()
    mainWindow.restoreState(b)
    if 'treeView' in globals() and userSettings.contains('TreeViewState/state'):
        tvh=treeView.header()
        if tvh:
            tvh.restoreState(userSettings.value('TreeViewState/state'))


def consoleCustomContextMenuRequested(pos):
    if not console: return
    standart_menu = console.createStandardContextMenu()
    if standart_menu:
        a = standart_menu.addAction(u'Очистить')
        a.connect('triggered()', console, 'clear()')
        getattr(standart_menu, 'exec')(console.mapToGlobal(pos))
        standart_menu.deleteLater()

def pythonConsoleCustomContextMenuRequested(pos):
    if not pythonConsole: return
    standart_menu = pythonConsole.createStandardContextMenu()
    if standart_menu:
        a = standart_menu.addAction(u'Очистить')
        a.connect('triggered()', pythonConsole, 'clear()')
        getattr(standart_menu, 'exec')(pythonConsole.mapToGlobal(pos))
        standart_menu.deleteLater()


def treeViewActivated():
    i = treeView.currentIndex()
    if not i.model().rowCount(i): return
    tableView.setRootIndex(
        tableView.model().mapFromSource(treeView.model().mapToSource(i))
    )
    tableView.resizeRowsToContents()
    tableView.resizeColumnsToContents()
    s = ''
    while i.isValid():
        ss = '/%s' % i.data()
        p = ss.find('(')
        if p >= 0: ss = ss[:p]
        s = ss + s
        i = i.parent()
    tableView.parentWidget().parentWidget().windowTitle = s


def setupUi(uiFileName):
    from RoadCentre import kernel

    global osmView
    global osmView2
    global imageView
    global console
    global mainWindow
    global treeView
    global tableView
    global listView

    for w in QtGui.QApplication.topLevelWidgets():
        if w.objectName == 'MainWindow': mainWindow = w
    else:
        mainWindow = widgetFromUi(uiFileName)

    if not mainWindow: return False

    treeView = mainWindow.findChild(QtGui.QTreeView)
    kernel.createSelectionModel(treeView)
    tableView = mainWindow.findChild(QtGui.QTableView)
    kernel.createSelectionModel(tableView)
    listView = mainWindow.findChild(QtGui.QListView,'listView')
    # kernel.createSelectionModel(listView)
    listView.setModel(kernel.itemModel)
    listView.setSelectionModel(kernel.selectionModel)

    treeView.connect('activated(const QModelIndex&)', treeViewActivated)

    mainWindow.statusBar().addPermanentWidget(QtGui.QLabel())
    for w in mainWindow.findChildren(QtGui.QGraphicsView):
        w.setScene(kernel.graphicsScene)
        w.connect('mouseMove(QPointF)', graphicsViewMouseMove)

    d = QtGui.QDockWidget(mainWindow.tr('Python console'), mainWindow)
    d.objectName = "pythonConsole"
    mainWindow.addDockWidget(QtCore.Qt.BottomDockWidgetArea, d)
    d.setWidget(pythonConsole)

    osmView = mainWindow.findChild(QtGui.QGraphicsView, 'graphicsView')
    imageView = osmView2 = None
    w = mainWindow.findChild(QtGui.QGraphicsView, 'graphicsView_2')
    if w:
        if w.className() == 'ImageView':
            imageView = w
        elif w.className() == 'OsmView':
            osmView2 = w

    console = mainWindow.findChild(QtGui.QPlainTextEdit, "plainTextEdit")
    if console:
        console.connect('customContextMenuRequested(QPoint)', consoleCustomContextMenuRequested)
        console.setContextMenuPolicy(QtCore.Qt.CustomContextMenu);
        console.setWordWrapMode(QtGui.QPlainTextEdit.NoWrap)
    if pythonConsole:
        pythonConsole.connect('customContextMenuRequested(QPoint)', pythonConsoleCustomContextMenuRequested)
        pythonConsole.setContextMenuPolicy(QtCore.Qt.CustomContextMenu);

    return True


def projectLoaded():
    from RoadCentre import project, kernel
    import os
    # tableView.setRootIndex(tableView.model().mapFromSource(kernel.itemModel.index(0,0)))
    # tableView.resizeRowsToContents()
    # tableView.resizeColumnsToContents()
    listView = mainWindow.findChild(QtGui.QListView,'listView')
    # listView.setRootIndex(listView.model().mapFromSource(kernel.itemModel.index(0,0)))
    listView.setRootIndex(kernel.itemModel.index(0, 0))

    maps_dir = os.path.join(project.baseDir, 'maps')
    if not os.path.exists(maps_dir):
        os.mkdir(maps_dir)

    osmView.operationWidget.clearLayers()
    if osmView2: osmView2.operationWidget.clearLayers()
    maps = []
    for i in range(project.settings.beginReadArray('Map')):
        project.settings.setArrayIndex(i)
        osmView.operationWidget.addMapLayer(
            project.settings.value('name'),
            QtCore.QDir(project.baseDir).filePath(project.settings.value('dbFile')),
            project.settings.value('tileServer')
        )
        if osmView2: osmView2.operationWidget.addMapLayer(
            project.settings.value('name'),
            QtCore.QDir(project.baseDir).filePath(project.settings.value('dbFile')),
            project.settings.value('tileServer')
        )
    project.settings.endArray()
    osmView.operationWidget.updateLayers()
    if osmView2: osmView2.operationWidget.updateLayers()

    if imageView:
        imageView.zoom = 1
        imageView.centerOn(kernel.graphicsScene.sceneRect.center())


def changeDefaultLoaded(checked):
    from RoadCentre import project
    from settings import userSettings
    def_project = None
    if checked:
        def_project = project.fileName
    userSettings.setValue('Default/DefaultProject', def_project)
    userSettings.sync()


def saveLayersState():
    from RoadCentre import kernel, project
    from settings import userSettings
    userSettings.beginGroup('Layers')
    for l in kernel.graphicsScene.layers():
        userSettings.setValue(l.name, (l.z, int(l.visible)))
    userSettings.endGroup()
    userSettings.sync()


def restoreLayersState():
    from RoadCentre import kernel, project
    from settings import userSettings
    userSettings.beginGroup('Layers')
    for l_name in userSettings.allKeys():
        layer = kernel.graphicsScene.layer(l_name)
        if not layer:
            continue
        val = userSettings.value(l_name)
        try:
            layer.z = float(val[0])
            layer.visible = int(val[1])
        except:
            pass
    userSettings.endGroup()

def showSelectionOnTree():
    if 'treeView' not in globals():
        msgError(u"Не определён объект 'treeView'")
        return
    for idx in treeView.selectedIndexes():
        parent_idx = idx.parent()
        if parent_idx.isValid() and not treeView.isExpanded(parent_idx):
            treeView.expand(parent_idx)
        treeView.scrollTo(idx)

