# -*- coding: utf-8 -*-

from PythonQt import *
import RoadCentre
from gui import treeView
if hasattr(QtCore, 'QObject'):
    from QtCore import QObject
elif hasattr(private, 'QObject'):
    QObject = private.QObject

class Menu():
    def __init__(self, kernel):
        self.selection = kernel.selection
        if hasattr(kernel,'graphicsScene'):
            self.sceneSelection = kernel.graphicsScene.selectedItems
        self.__menu = Qt.QMenu()

    def menuRequested(self, pos):
        menus = []
        menuObjects={}
        selections=self.selection()
        if len(selections)==0 and hasattr(self,'sceneSelection'):
            selections=self.sceneSelection()
        for dev in selections:
            obj_cls = None
            if not dev.inherits('QObject') and not isinstance(dev,QObject):
                if not dev.inherits('QGraphicsItem'):
                    continue
                obj_cls = 'QGraphicsItem%d' % dev.type()
            if obj_cls is None:
                obj_cls=RoadCentre.kernel.objectClassName(dev)
            if obj_cls not in self.devClassMenus:
                obj_cls=dev.className()
            if obj_cls in self.devClassMenus:
                if obj_cls not in menuObjects: menuObjects[obj_cls]=(self.devClassMenus[obj_cls],[])
                menuObjects[obj_cls][1].append(dev)
        for s in menuObjects.values():
            menuObj, obj = s
            m = menuObj.menu(obj)
            if m: menus.append(m)

        def add_collapse_all_action(_m): _m.addAction(u"Свернуть всё").connect('triggered()', treeView.collapseAll)
        can_collapse_all = treeView.rect.contains(treeView.mapFromGlobal(QtGui.QCursor.pos()))

        if len(menus) == 0:
            if can_collapse_all:
                self.__menu.clear()
                add_collapse_all_action(self.__menu)
                self.__menu.popup(QtGui.QCursor.pos())
            return
        if len(menus) == 1:
            if can_collapse_all:
                add_collapse_all_action(menus[0])
            menus[0].popup(QtGui.QCursor.pos())
            return
        self.__menu.clear()
        for m in menus: self.__menu.addMenu(m)
        if can_collapse_all:
            add_collapse_all_action(self.__menu)

        self.__menu.popup(QtGui.QCursor.pos())

    def setWidgetContextMenu(self, w):
        w.contextMenuPolicy=QtCore.Qt.CustomContextMenu
        w.connect('customContextMenuRequested(const QPoint &)', self.menuRequested)

    devClassMenus={}

class SubMenu:
    def __init__(self, titles = ('','')):
        self.menuMain = Qt.QMenu()
        self.menuCustom = Qt.QMenu()
        self.__menu = Qt.QMenu()
        self.selection = ()
        self.titles = titles

    def menu(self, obj):
        menu = self.__menu
        menu.clear()
        menu.insertActions(0, self.menuMain.actions())
        menu.insertActions(0, self.menuCustom.actions())
        self.selection=obj
        if (len(obj) == 1):
            if hasattr(obj[0],'customMenu'): menu.addMenu(obj[0].customMenu)
            menu.title = self.titles[0]
        else: menu.title = self.titles[1]
        return menu

    def appendCustomAction(self, name = None, fun = None):
        if name:
            for a in self.menuCustom.findChildren('QAction'):
                if a.text == name:
                    return
            a=self.menuCustom.addAction(name)
            if fun:
                a.connect('triggered()', fun)
                try:
                    fun.sender=self
                except AttributeError:
                    pass
        else: a=self.menuCustom.addSeparator()
        return a
    
    def appendSubmenuAction(self, submenu_name,name = None, fun = None):
        for m in self.menuCustom.findChildren('QMenu'):
            if m.title == submenu_name:
                if name:
                    a=m.addAction(name)
                    if fun:
                        a.connect('triggered()', fun)
                        fun.sender=self
                else: a=m.addSeparator()
                return a

    def appendCustomMenu(self, name):
        return self.menuCustom.addMenu(name)

    def removeCustomAction(self, name):
        for a in self.menuCustom.findChildren('QAction'):
            if a.text == name:
                a.delete()
                break

    @staticmethod
    def checkAction(act,default_protocol='Spectr'):
        if not hasattr(RoadCentre.project.connection,'commandAvailable'):
            return True
        data=act.data()
        if data==None:
            return True
        if type(data)!=tuple: data=(data,)
        act_enabled=True
        for cmd in data:
            if type(cmd)!=str or len(cmd)==0:
                continue
            cmd_list=cmd.split('/')
            if len(cmd_list)==1:
                cmd_list.insert(0,default_protocol)
            act_enabled &= RoadCentre.project.connection.commandAvailable(cmd_list[0],cmd_list[1])
            if not act_enabled:
                break
        return act_enabled
